searchData={"items":[{"type":"behaviour","title":"Dotenvy","doc":"`Dotenvy` is an Elixir port of the original [dotenv](https://github.com/bkeepers/dotenv) Ruby gem.\n\nIt is designed to help applications follow the principles of\nthe [12-factor app](https://12factor.net/) and its recommendation to store\nconfiguration in the environment.\n\nUnlike other configuration helpers, `Dotenvy` enforces no convention for the naming\nof your files: `.env` is a common choice, you may name your configuration files whatever\nyou wish.\n\nSee the [Getting Started](docs/getting_started.md) page for more info.","ref":"Dotenvy.html"},{"type":"function","title":"Dotenvy.env!/2","doc":"Reads the given env `variable` and converts its value to the given `type`.\n\nThis function attempts to read a value from a local data store of sourced values.\n\nThis function may raise an error because type conversion is delegated to\n`Dotenvy.Transformer.to!/2` -- see its documentation for a list of supported types.","ref":"Dotenvy.html#env!/2"},{"type":"function","title":"Examples - Dotenvy.env!/2","doc":"iex> env!(\"PORT\", :integer)\n    5432\n    iex> env!(\"ENABLED\", :boolean)\n    true","ref":"Dotenvy.html#env!/2-examples"},{"type":"function","title":"Dotenvy.env!/3","doc":"Reads an env variable and converts its output or returns a default value.\n\n> #### Use `env!/2` when possible {: .info}\n>\n> `env!/2` is recommended over `env!/3` because it creates a stronger contract with\n> the environment: your app literally will not start when required env variables are missing.\n\nIf the given `variable` is *set*, its value is converted to the given `type`.\nThe provided `default` value is _only_ used when the variable is _not_ set\n(i.e. when it does not exist).\n\n**The `default` value is returned as-is, without conversion**. This allows\ngreater control of the output.\n\nConversion is delegated to `Dotenvy.Transformer.to!/2`, which may raise an error.\nSee its documentation for a list of supported types.\n\nThis function attempts to read a value from a local data store of sourced values.","ref":"Dotenvy.html#env!/3"},{"type":"function","title":"Examples - Dotenvy.env!/3","doc":"iex> env!(\"PORT\", :integer, 5432)\n    5433\n\n    iex> env!(\"NOT_SET\", :boolean, %{not: \"converted\"})\n    %{not: \"converted\"}\n\n    iex> System.put_env(\"HOST\", \"\")\n    iex> env!(\"HOST\", :string!, \"localhost\")\n    ** (RuntimeError) Error converting HOST to string!: non-empty value required","ref":"Dotenvy.html#env!/3-examples"},{"type":"type","title":"Dotenvy.input_source/0","doc":"An input source may be either a path to an env file or a map with string keys\nand values, e.g. `\"envs/.env\"` or `%{\"FOO\" => \"bar\"}`. This allows users to\nspecify a list of env files interspersed with other values from other sources,\nmost commonly `System.get_env()`.","ref":"Dotenvy.html#t:input_source/0"},{"type":"callback","title":"Dotenvy.parse/3","doc":"A parser implementation should receive the `contents` read from a file,\na map of `vars` (with string keys, as would come from `System.get_env/0`),\nand a keyword list of `opts`.\n\nSee `Dotenvy.Parser` for the default implementation of this callback.","ref":"Dotenvy.html#c:parse/3"},{"type":"function","title":"Dotenvy.source/2","doc":"Like its Bash namesake command, `source/2` accumulates values from the given input(s).\nThe accumulated values are stored via a side effect function to make them available\nto the `env!/2` and `env!/3` functions.\n\nThink of `source/2` as a _merging operation_ which can accept maps (like `Map.merge/2`)\nor paths to env files.\n\nInputs are processed from left to right so that values can be overridden by each\nsubsequent input. As with `Map.merge/2`, the right-most input takes precedence.","ref":"Dotenvy.html#source/2"},{"type":"function","title":"Options - Dotenvy.source/2","doc":"- `:parser` module that implements `c:Dotenvy.parse/3` callback. Default: `Dotenvy.Parser`\n\n- `:require_files` specifies which of the given `files` (if any) *must* be present.\n  When `true`, all the listed files must exist.\n  When `false`, none of the listed files must exist.\n  When some of the files are required and some are optional, provide a list\n  specifying which files are required. If a file listed here is not included\n  in the function's `files` argument, it is ignored. Default: `false`\n\n- `:side_effect` an arity 1 function called after successfully parsing inputs.\n  The default is an internal function that stores the values inside a process dictionary so\n  the values are available to the `env!/2` and `env!/3` functions. This option\n  is overridable to facilitate testing. Changing it is not recommended.","ref":"Dotenvy.html#source/2-options"},{"type":"function","title":"Examples - Dotenvy.source/2","doc":"The simplest implementation is to parse a single file by including its path:\n\n    iex> Dotenvy.source(\".env\")\n    {:ok, %{\n      \"TIMEOUT\" => \"5000\",\n      \"DATABASE_URL\" => \"postgres://postgres:postgres@localhost/myapp\",\n      # ...etc...\n      }\n    }\n\nMore commonly, you will source _multiple_ files (often based on the `config_env()`)\nand you will defer to pre-existing system variables. The most common pattern looks like this:\n\n      iex> Dotenvy.source([\n        \"#{config_env()}.env\",\n        \"#{config_env()}.override.env\",\n        System.get_env()\n      ])\n\nIn the above example, the `prod.env`, `dev.env`, and `test.env` files would be version-controlled,\nbut the `*.override.env` variants would be ignored, giving developers the ability to override\nvalues without needing to modify versioned files.\n\n> #### Give Precedence to System Envs! {: .warning}\n>\n> Don't forget to include `System.get_env()` as the final input to `source/2` so that\n> system environment variables take precedence over values sourced from `.env` files.\n>\n> When you run a shell command like `❯ LOG_LEVEL=debug mix run`, your expectation is probably that\n> the `LOG_LEVEL` variable would be set to `debug`, overriding whatever may have been defined\n> in your sourced `.env` files. Similarly, you may export env vars in your Bash profile.\n> System env vars are not granted precedence automatically: you must explicitly include\n> `System.get_env()` as the final input to `source/2`.\n\n\nIf your env files are making use of variable substitution based on system env vars,\ne.g. `${PWD}` (see the [Dotenv File Format](docs/dotenv-file-format.md)), then you\nwould need to specify `System.get_env()` as the first argument to `source/2`.\n\nFor example, if your `.env` references the system `HOME` variable:\n  ```\n  # .env\n  CACHE_DIR=${HOME}/cache\n  ```\n\nthen your `source/2` command would need to make the system env vars available\nto the parser by including them as one of the inputs, e.g.\n\n      iex> Dotenvy.source([System.get_env(), \".env\"])\n\nIn some cases, you may wish to reference the system vars both _before and after_\nyour own .env files, e.g.\n\n      iex> Dotenvy.source([System.get_env(), \".env\", System.get_env()])\n\nor you may wish to cherry-pick which variables you need to make available for\nvariable substitution:\n\n      iex> Dotenvy.source([\n        %{\"HOME\" => System.get_env(\"HOME\")},\n        \".env\",\n        System.get_env()\n      ])\n\nThis syntax favors explicitness so there is no confusion over what might have been\n\"automagically\" accumulated.\n\nIf you need to make any variables parsed from your files available elsewhere in\nthe application via `System.get_env/2`, then you can call `System.put_env/1` on\nthe output of `Dotenvy.source/2`, e.g.\n\n    iex> {:ok, parsed_vars} = Dotenvy.source([\".env\", System.get_env()])\n    iex> System.put_env(parsed_vars)","ref":"Dotenvy.html#source/2-examples"},{"type":"function","title":"Dotenvy.source!/2","doc":"As `source/2`, but returns a map on success or raises on error.","ref":"Dotenvy.html#source!/2"},{"type":"module","title":"Dotenvy.Parser","doc":"This module handles the parsing of the contents of `.env` files into maps with\nstring keys. See [Dotenv File Format](docs/dotenv-file-format.md) for details\non the supported file format.\n\nThis implementation uses parsing over regular expressions for most of its work.","ref":"Dotenvy.Parser.html"},{"type":"function","title":"Dotenvy.Parser.parse/3","doc":"Parse the given `contents`, substituting and merging with the given `vars`.","ref":"Dotenvy.Parser.html#parse/3"},{"type":"module","title":"Dotenvy.Transformer","doc":"This module provides functionality for converting string values to specific Elixir data types.\n\nThese conversions were designed to operate on system environment variables, which\n_always_ store string binaries.","ref":"Dotenvy.Transformer.html"},{"type":"type","title":"Dotenvy.Transformer.conversion_type/0","doc":"The conversion type specifies the target data type to which a string will be converted.\nFor example, `:integer` would indicate a transformation of `\"12\"` to `12`.\n\nThe following types are supported:\n\n- `:atom` - converts to an atom. An empty string will be the atom `:\"\"` (!).\n- `:atom?` - converts to an atom. An empty string will be considered `nil`\n- `:atom!` - converts to an atom. An empty string will raise.\n\n- `:boolean` - \"false\", \"0\", or an empty string \"\" will be considered boolean `false`. Any other non-empty value is considered `true`.\n- `:boolean?` - as above, except an empty string will be considered `nil`\n- `:boolean!` - as above, except an empty string will raise.\n\n- `:charlist` - converts string to charlist.\n- `:charlist?` - converts string to charlist. Empty string will be considered `nil`.\n- `:charlist!` - as above, but an empty string will raise.\n\n- `:integer` - converts a string to an integer. An empty string will be considered `0`.\n- `:integer?` - as above, but an empty string will be considered `nil`.\n- `:integer!` - as above, but an empty string will raise.\n\n- `:float` - converts a string to an float. An empty string will be considered `0`.\n- `:float?` - as above, but an empty string will be considered `nil`.\n- `:float!` - as above, but an empty string will raise.\n\n- `:existing_atom` - converts into an existing atom. Raises error if the atom does not exist.\n- `:existing_atom?` - as above, but an empty string will be considered `nil`.\n- `:existing_atom!` - as above, but an empty string will raise.\n\n- `:module` - converts a string into an Elixir module name. Raises on error.\n- `:module?` - as above, but an empty string will be considered `nil`.\n- `:module!` - as above, but an empty string will raise.\n\n- `:string` - no conversion (default)\n- `:string?` - empty strings will be considered `nil`.\n- `:string!` - as above, but an empty string will raise.\n- custom function - see below.","ref":"Dotenvy.Transformer.html#t:conversion_type/0"},{"type":"type","title":"Custom Callback function - Dotenvy.Transformer.conversion_type/0","doc":"When you require more control over the transformation of your value than is possible\nwith the types provided, you can provide an arity 1 function in place of the type.","ref":"Dotenvy.Transformer.html#t:conversion_type/0-custom-callback-function"},{"type":"function","title":"Dotenvy.Transformer.to!/2","doc":"Converts strings into Elixir data types with support for nil-able values. Raises on error.\n\nEach type determines how to interpret the incoming string, e.g. when the `type`\nis `:integer`, an empty string is considered a `0`; when `:integer?` is the `type`,\nand empty string is converted to `nil`.\n\nRemember:\n\n- The type should use a `?` suffix when an empty string should be considered `nil` (a.k.a. a \"nullable\" value).\n- The type should a `!` suffix when an empty string is not allowed. Use this when values are required.","ref":"Dotenvy.Transformer.html#to!/2"},{"type":"function","title":"Types - Dotenvy.Transformer.to!/2","doc":"See the `t:Dotenvy.Transformer.conversion_type/0` for a description of valid\nconversion types.","ref":"Dotenvy.Transformer.html#to!/2-types"},{"type":"function","title":"Examples - Dotenvy.Transformer.to!/2","doc":"iex> to!(\"debug\", :atom)\n    :debug\n    iex> to!(\"\", :boolean)\n    false\n    iex> to!(\"\", :boolean?)\n    nil\n    iex> to!(\"5432\", :integer)\n    5432\n    iex> to!(\"DateTime\", :module)\n    DateTime\n    iex> to!(\"foo\", fn val -> val <> \"bar\" end)\n    \"foobar\"\n    iex> Dotenvy.Transformer.to!(\"Oops\", :float)\n      ** (Dotenvy.Error) Unparsable as float\n      (dotenvy 1.0.0) lib/dotenvy/transformer.ex:165: Dotenvy.Transformer.to!/2","ref":"Dotenvy.Transformer.html#to!/2-examples"},{"type":"exception","title":"Dotenvy.Error","doc":"This error module can be useful when writing your own custom conversion\nfunctions because special contextual information will be included with any\nerrors.","ref":"Dotenvy.Error.html"},{"type":"exception","title":"Examples - Dotenvy.Error","doc":"Let's say your configuration needs to supply one of a set of possible values\n(i.e. an enum). We can define a custom function to support this and pass it\nas the second argument to `Dotenvy.env!/2`\n\n    # runtime.exs\n    import Config\n    import Dotenvy\n\n    size_enum = fn\n      \"large\" -> :large\n      \"small\" -> :small\n      _ ->\n        raise Dotenvy.Error, message: \"allowed size_enum values are large or small\"\n    end\n\n    config :myapp, :some_bool, env!(\"SIZE\", size_enum)","ref":"Dotenvy.Error.html#module-examples"},{"type":"extras","title":"Dotenvy","doc":"# Dotenvy\n\n[![Module Version](https://img.shields.io/hexpm/v/dotenvy.svg)](https://hex.pm/packages/dotenvy)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/dotenvy/)\n[![Total Download](https://img.shields.io/hexpm/dt/dotenvy.svg)](https://hex.pm/packages/dotenvy)\n[![License](https://img.shields.io/hexpm/l/dotenvy.svg)](https://hex.pm/packages/dotenvy)\n[![Last Updated](https://img.shields.io/github/last-commit/fireproofsocks/dotenvy.svg)](https://github.com/fireproofsocks/dotenvy/commits/master)\n\n`Dotenvy` is an Elixir port of the original [dotenv](https://github.com/bkeepers/dotenv) Ruby gem, **compatible with mix and releases**. It is designed to help the development of applications following the principles of the [12-factor app](https://12factor.net/) and its recommendation to store configuration in the environment.","ref":"readme.html"},{"type":"extras","title":"Installation - Dotenvy","doc":"Add `dotenvy` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:dotenvy, \"~> 1.0.0\"}\n  ]\nend\n```\n\nIt has no dependencies.","ref":"readme.html#installation"},{"type":"extras","title":"Usage - Dotenvy","doc":"`Dotenvy` is designed to help configure your application _at runtime_, and one\nof the most effective places to do that is inside `config/runtime.exs` (available\nsince Elixir v1.11).\n\nThe `Dotenvy.source/2` function can accept a single file or a list of files.  When combined with `Config.config_env/0` it is easy to load up environment-specifc config. A common setup in your `config/runtime.exs` would include a block like the following:\n\n```elixir\n# config/runtime.exs\nimport Config\nimport Dotenvy\n\nenv_dir_prefix = System.get_env(\"RELEASE_ROOT\") || Path.expand(\"./envs/\")\n\nsource!([\n  Path.absname(\".env\", env_dir_prefix),\n  Path.absname(\".#{config_env()}.env\", env_dir_prefix),\n  Path.absname(\".#{config_env()}.overrides.env\", env_dir_prefix),\n  System.get_env()\n])\n```\n\nThe above example would include the `envs/.env` file for shared/default configuration values and then leverage environment-specific `.env` files (e.g. `envs/.dev.env`) to provide environment-specific values. An `envs/.{MIX_ENV}.overrides.env` file would be referenced in the `.gitignore` file to allow developers to override any values a file that is not under version control. System environment variables are given final say over the values via `System.get_env()`.  Think of `Dotenvy.source/2` as a _merge operation_, similar to `Map.merge/2`: the last input takes precedence.\n\nBy default, the listed files do not _need_ to exist (you can leverage the `:require_files` option if needed). The `Dotenvy.source/2` function only needs to know where to look. This makes it easy to commit default values while still leaving the door open to developers to override values via their own configuration files.\n\nYou control if and how existing system env vars are handled: usually they should take precedence over values defined in `.env` files, so for most apps, the `System.get_env()` should be included as the final input supplied to `source/2`.\n\nUnlike other packages, `Dotenvy` has no opinions about the names or locations of your `.env` files, you just need to pass their paths to `Dotenvy.source/2` or `Dotenvy.source!/2`.\n\nFor a simple example, we can load up a `.env` file containing all defaults and an environment-specific file (e.g. `.dev.env`).  Remember to use both [`Path.expand/1`](https://hexdocs.pm/elixir/Path.html#expand/1) and [`Path.absname/2`](https://hexdocs.pm/elixir/Path.html#absname/2) so that the operating system can resolve your file names into fully qualified paths that work when you are developing locally or when your app is running as a release or inside a Livebook.\n\n```elixir\n# config/runtime.exs\nimport Config\nimport Dotenvy\n\nenv_dir_prefix = System.get_env(\"RELEASE_ROOT\") || Path.expand(\"./envs/\")\n\nsource!([\n  Path.absname(\".env\", env_dir_prefix), \n  Path.absname(\"#{config_env()}.env\", env_dir_prefix), \n  System.get_env()\n  ])\n\nconfig :myapp, MyApp.Repo,\n    database: env!(\"DATABASE\", :string!),\n    username: env!(\"USERNAME\", :string),\n    password: env!(\"PASSWORD\", :string),\n    hostname: env!(\"HOSTNAME\", :string!),\n    pool_size: env!(\"POOL_SIZE\", :integer),\n    adapter: env!(\"ADAPTER\", :module, Ecto.Adapters.Postgres),\n    pool: env!(\"POOL\", :module?)\n```\n\nAnd then define your variables in the file(s) to be sourced. `Dotenvy` has no opinions about what you name your files; `.env` is merely a convention.\n\n```env\n# .env\nDATABASE=myapp_dev\nUSERNAME=myuser\nPASSWORD=mypassword\nHOSTNAME=localhost\nPOOL_SIZE=10\nPOOL=\n```\n\nWhen you set up your application configuration in this way, you are creating a contract with the environment: `Dotenvy.env!/2` will raise if the required variables have not been set or if the values cannot be properly transformed. This is an approach that works equally well for your day-to-day development and for mix releases.\n\nRead the [Getting Started guide](docs/getting_started.md) for more details.\n\nRefer to the [\"dotenv\" (`.env`) file format](docs/reference/dotenv-file-format.md) for more examples and features of the supported syntax.\n\nSee the `Dotenvy` module documentation on its functions.","ref":"readme.html#usage"},{"type":"extras","title":"Note for Mix Tasks - Dotenvy","doc":"If you have authored your own Mix tasks, you must ensure that they load the\napplication configuration in a way that is compatible with the runtime config.\nA good way to do this is to include `Mix.Task.run(\"app.config\")` inside the\n`run/1` implementation, e.g.\n\n```elixir\ndef run(_args) do\n  Mix.Task.run(\"app.config\")\n  # ...\nend\n```\n\nIf you are dealing with third-party mix tasks that fail to properly load configuration, you may need to manually call `mix app.config` before running them, e.g.\n\n```sh\nmix do app.config other.task\n```\n\nDefining a task `alias` in `mix.exs` is another way to accomplish this:\n\n```elixir\n# mix.exs\ndefp aliases do\n    [\n      \"other.task\": [\"app.config\", \"other.task\"]\n    ]\n```","ref":"readme.html#note-for-mix-tasks"},{"type":"extras","title":"Upgrading from v0.5.0 or before - Dotenvy","doc":"Starting with `Dotenvy` v0.6.0, the precedence of system env variables over parsed `.env` files is not defined; the `:overwrite?` and `:vars` options are no longer supported in `Dotenvy.source/2` and `Dotenvy.source!/2`. Instead, the `source` functions now accept file paths OR maps: this makes the question of variable precedence something that must be explicitly listed. The `source` functions act more like `Map.merge/2`, accumulating values, always giving precedence to the righthand source.\n\nMost users upgrading from v0.5.0 will wish to include `System.get_env()` as the final input to `source/2`.\n\n```elixir\n# in dotenvy 0.5.0 or before:\nsource([\"#{config_env()}.env\", \"#{config_env()}.override.env\"])\n\n# should be changed to the following in dotenvy 0.6.0:\nsource([\"#{config_env()}.env\", \"#{config_env()}.override.env\", System.get_env()])\n```\n\nIf you are relying on [variable interpolation](docs/reference/dotenv-file-format.md) in your `.env` files, you may also need to include `System.get_env()` (or an equivalent subset) _before_ you list your `.env` files.  This is necessary to make values available to the file parser.\n\n```elixir\n# in dotenvy 0.5.0 or before:\nsource([\"#{config_env()}.env\", \"#{config_env()}.override.env\"])\n\n# should be changed to the following in dotenvy 0.6.0:\nsource([System.get_env(), \"#{config_env()}.env\", \"#{config_env()}.override.env\", System.get_env()])\n```\n\nThe change in syntax introduced in v0.6.0 favors a declarative list of sources over opaquely inferred inputs. This also opens the door for compatibility with other value sources, e.g. secure parameter stores.\n\n---------------------------------------------------\n\nImage Attribution: artwork by Beck","ref":"readme.html#upgrading-from-v0-5-0-or-before"},{"type":"extras","title":"Cheatsheet","doc":"# Cheatsheet\n\nQuick reference for using `Dotenvy`.","ref":"cheatsheet.html"},{"type":"extras","title":"Setup - Cheatsheet","doc":"{: .col-3}","ref":"cheatsheet.html#setup"},{"type":"extras","title":"envs/ Directory - Cheatsheet","doc":"The recommended location for storing your `.env` files is inside a dedicated `envs/` directory.\nCode editors recognize the `.env` extension.\n\n#### Example envs/dev.env\n\n```env\nAPI_KEY=xyz\nAWS_REGION=\"us-east-1\"\nHTTP_CLIENT=HTTPoison\n```","ref":"cheatsheet.html#envs-directory"},{"type":"extras","title":"Version Control - Cheatsheet","doc":"`envs/dev.env` is tracked by Git. Use it to store sensible non-sensitive defaults.\n\n`envs/dev.overrides.env` is ignored by Git. Use it to override any variables in the tracked version.\n\nThis must match the values used in the runtime configuration.\n\n#### .gitignore\n\n```\n*.overrides.env\n```","ref":"cheatsheet.html#version-control"},{"type":"extras","title":"Release Compatibility - Cheatsheet","doc":"For compatibility with [releases](https://hexdocs.pm/mix/Mix.Release.html), \nconfigure your builds to copy (i.e. \"overlay\") the contents of your `envs/` \ndirectory into the root of the release.\n\n#### mix.exs\n\n```elixir\ndefp releases do\n[\n    my_app: [\n      overlays: [\"envs/\"]\n    ]\n]\nend\n```","ref":"cheatsheet.html#release-compatibility"},{"type":"extras","title":"Runtime Config - Cheatsheet","doc":"Your `config/runtime.exs` is where you source your environment variables. \nThe last argument to `Dotenvy.source/2` or `Dotenvy.source!/2` takes precedence.\nIt's common to use `System.get_env()` as the final argument so any existing system\nenvironment variables will take precedence over anything parsed from the `.env` files.\n\n#### config/runtime.exs\n\n```elixir\nimport Config\nimport Dotenvy\n\nenv_dir = System.get_env(\"RELEASE_ROOT\") || Path.expand(\"./envs/\")\n  \nsource!([\n    Path.absname(\".env\", env_dir),\n    Path.absname(\".#{config_env()}.env\", env_dir),\n    Path.absname(\".#{config_env()}.overrides.env\", env_dir),\n    System.get_env()\n  ])\n```","ref":"cheatsheet.html#runtime-config"},{"type":"extras","title":"Example Database Configuration - Cheatsheet","doc":"{: .col-3}","ref":"cheatsheet.html#example-database-configuration"},{"type":"extras","title":"Dev ENV - Cheatsheet","doc":"#### envs/dev.env\n\n```\nPG_USERNAME=postgres\nPG_PASSWORD=postgres\nPG_HOSTNAME=localhost\nPG_PORT=5432\nPG_DATABASE=m_app_dev\nPG_POOL_SIZE=10\nPG_POOL=DBConnection.ConnectionPool\nPG_SSL=true\n```","ref":"cheatsheet.html#dev-env"},{"type":"extras","title":"Test ENV - Cheatsheet","doc":"#### envs/test.env\n\n```\nPG_USERNAME=postgres\nPG_PASSWORD=postgres\nPG_HOSTNAME=localhost\nPG_PORT=5432\nPG_DATABASE=my_app_test\nPG_POOL_SIZE=10\nPG_POOL=Ecto.Adapters.SQL.Sandbox\nPG_SSL=false\n```","ref":"cheatsheet.html#test-env"},{"type":"extras","title":"Runtime Configuration - Cheatsheet","doc":"#### config/runtime.exs\n\n```\nimport Config\nimport Dotenvy\n\nenv_dir = System.get_env(\"RELEASE_ROOT\") || Path.expand(\"./envs/\")\n  \nsource!([\n    Path.absname(\".env\", dir),\n    Path.absname(\".#{config_env()}.env\", env_dir),\n    Path.absname(\".#{config_env()}.overrides.env\", env_dir),\n    System.get_env()\n  ])\n\nconfig :my_app, MyApp.PGRepo,\n  pool: env!(\"PG_POOL\", :module?),\n  pool_size: env!(\"PG_POOL_SIZE\", :integer),\n  database: env!(\"PG_DATABASE\", :string),\n  username: env!(\"PG_USERNAME\", :string),\n  password: env!(\"PG_PASSWORD\", :string),\n  port: env!(\"PG_PORT\", :integer),\n  hostname: env!(\"PG_HOSTNAME\", :string)\n```","ref":"cheatsheet.html#runtime-configuration"},{"type":"extras","title":"Transformations - Cheatsheet","doc":"System Environment variables are always stored as strings which may need to be \ntransformed into native Elixir data types.\n\n#### Used as the 2nd argument to `Dotenvy.env!/2` and `Dotenvy.env!/3`\n\n| Conversion Type   | Elixir Type   | On Empty String   |\n| --                | --            | --                |\n| `:atom`           | atom          | `:\"\"`             |\n| `:atom?`          | atom          | `nil`             |\n| `:atom!`          | atom          | raise ⚠           |\n| `:boolean`        | boolean       | `false`           |\n| `:boolean?`       | boolean       | `nil`             |\n| `:boolean!`       | boolean       | raise ⚠           |\n| `:charlist`       | charlist      | `''` i.e. `[]`    |\n| `:charlist?`      | charlist      | `nil`             |\n| `:charlist!`      | charlist      | raise ⚠           |\n| `:integer`        | integer       | `0`               |\n| `:integer?`       | integer       | `nil`             |\n| `:integer!`       | integer       | raise ⚠           |\n| `:float`          | float         | `0`               |\n| `:float?`         | float         | `nil`             |\n| `:float!`         | float         | raise ⚠           |\n| `:existing_atom`  | atom          | `:\"\"` or raise    |\n| `:existing_atom?` | atom          | `nil`             |\n| `:existing_atom!` | atom          | raise ⚠           |\n| `:module`         | atom          | `:\"Elixir.\"`      |\n| `:module?`        | atom          | `nil`             |\n| `:module!`        | atom          | raise ⚠           |\n| `:string`         | String        | `\"\"`              |\n| `:string?`        | String        | `nil`             |\n| `:string!`        | String        | raise ⚠           |\n\nCustom functions handle their own behavior.\n\n#### Custom Function Example\n\n```env\nPHX_IP=\"0, 0, 0, 0, 0, 0, 0, 0\"\n```\n\n```elixir\nconfig :feenix, FeenixWeb.Endpoint,\n    http: [\n      # Enable IPv6 and bind on all interfaces.\n      ip: env!(\"PHX_IP\", fn ip -> \n        ip \n        |> String.split(\",\") \n        |> Enum.map(&String.trim/1) \n        |> Enum.map(&String.to_integer/1) \n        |> List.to_tuple()\n      end)\n    ],\n```\n\nYour custom functions can raise a `Dotenvy.Error` to benefit from improved messages\nthat include helpful context about any problems, e.g.\n\n```elixir\nstrict_boolean! = fn\n  \"true\" -> true\n  \"false\" -> false\n  _ ->\n    raise Dotenvy.Error,\n      message: \"strict_boolean! values must be either true or false\"\nend\n\nconfig :myapp, :some_bool, env!(\"SOME_BOOL\", strict_boolean!)\n```\n\nThis will yield an error like the following:\n\n```\n** (RuntimeError) Error converting variable SOME_BOOL using custom function: strict_boolean! values must be either true or false\n```","ref":"cheatsheet.html#transformations"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nThe format is based on [Keep a Changelog](https://keepachangelog.com/en/1.0.0/),\nand this project adheres to [Semantic Versioning](https://semver.org/spec/v2.0.0.html).","ref":"changelog.html"},{"type":"extras","title":"v1.0.0 - Changelog","doc":"- Adds support for shell commands via the `$()` syntax\n- new documentation pages added to introduce generators and more examples\n- Deletes deprecated `Dotenvy.env/3` function\n- Updates dependencies to latest\n- Better error messages for unset variables\n- Better error messages for `:require_files` errors\n- Updated Logo","ref":"changelog.html#v1-0-0"},{"type":"extras","title":"v0.9.0 - Changelog","doc":"- Removes fallback to `System.fetch_env/1` and instead requires explicit sourcing of system envs. See [Issue 21](https://github.com/fireproofsocks/dotenvy/issues/21)\n- Bumps Elixir version specified for local development in `.tool-versions`\n- Updates dependencies to latest","ref":"changelog.html#v0-9-0"},{"type":"extras","title":"v0.8.0 - Changelog","doc":"- Enables exception rescuing to report on problems with custom callback functions\n- Moves `Dotenvy.Transformer.Error` to `Dotenvy.Error` to offer a simpler interface\n  for devs who want to raise errors from custom transformer functions\n- Improved documentation and examples for usage in umbrella apps\n- Updates all internal options in the parser to use `%Opts{}` struct\n- Improves test coverage\n- Bumps Elixir version specified for local development in `.tool-versions`\n- Updates dependencies to latest","ref":"changelog.html#v0-8-0"},{"type":"extras","title":"v0.7.0 - Changelog","doc":"- Formally defines a type for all supported conversions to improve documentation and specs\n- Updates dependencies to latest\n- Specifies Elixir 1.13 as required (simply because I can't get anything older to compile)","ref":"changelog.html#v0-7-0"},{"type":"extras","title":"v0.6.0 - Changelog","doc":"- Does away with the confusing `:overwrite?` and `vars` options in favor of a simple declarative/explicit inputs. `source/2` now accepts ad-hoc maps as inputs.\n- Updates dependencies including `:ex_doc` to take advantage of admonishment blocks.\n- Various documentation cleanups/clarifications.","ref":"changelog.html#v0-6-0"},{"type":"extras","title":"v0.5.0 - Changelog","doc":"- Shifts storage of system environment variables to the application process dictionary and alters the reading of this data to help improve the security posture and avoid leaking env values. `:side_effect` option for `source/2` and `source!/2` function changed.","ref":"changelog.html#v0-5-0"},{"type":"extras","title":"v0.4.1 - Changelog","doc":"- Makes error messages more informative when unable to convert strings to integers or floats","ref":"changelog.html#v0-4-1"},{"type":"extras","title":"v0.4.0 - Changelog","doc":"- Adds support for custom transformer types by allowing an arity 1 function as the second argument to Dotenvy.Transformer.to/2. See [Issue 2](https://github.com/fireproofsocks/dotenvy/issues/2)","ref":"changelog.html#v0-4-0"},{"type":"extras","title":"v0.3.0 - Changelog","doc":"- Renames Dotenvy.Transformer.to/2 to `Dotenvy.Transformer.to!/2` to better communicate that it may raise an error.\n- Returns key name in errors for easier troubleshooting.\n- Tracks an error if the `:require_files` option lists a file not included in the `files` input (for sanity).\n- Introduces `Dotenvy.env!/3` (which is the same as `Dotenvy.env/3` but with no defaults provided). This better communicates that it may raise an error (because internally it relies on `Dotenvy.Transformer.to!/2`)\n- Deprecates `Dotenvy.env/3` in favor of `Dotenvy.env!/3`","ref":"changelog.html#v0-3-0"},{"type":"extras","title":"v0.2.0 - Changelog","doc":"Adds support for default `type` of `:string` to the `Dotenvy.env!/2` and `Dotenvy.env/3` functions.","ref":"changelog.html#v0-2-0"},{"type":"extras","title":"v0.1.0 - Changelog","doc":"Initial release.","ref":"changelog.html#v0-1-0"},{"type":"extras","title":"Getting Started","doc":"# Getting Started\n\nThe concept of environment variables is simple and `Dotenvy` aims to make your application take advantage of them, but how can you start using them easily in your application?  This page will walk you through kicking the tires of a simple application so you can learn how `Dotenvy` works.\n\n>","ref":"getting_started.html"},{"type":"extras","title":"Prerequisite {: .info} - Getting Started","doc":">\n> If you haven't already, [install the `dotenvy_generators`](docs/reference/generators.md).\n>\n> When you run `mix help`, you should see `dot.new` as one of the available tasks. Make sure that's available before continuing.","ref":"getting_started.html#prerequisite-info"},{"type":"extras","title":"Generating an app - Getting Started","doc":"The `dot.new` mix task is available when you have installed the [`dotenvy_generators`](docs/reference/generators.md). We can use it generate a new Elixir app:\n\n    mix dot.new example\n\nFollow the prompts given:\n\n    cd example\n    mix deps.get\n\nThe structure should look familiar to you -- the only thing you might notice is the presence of the `envs/` directory.","ref":"getting_started.html#generating-an-app"},{"type":"extras","title":"Environment-specific env files - Getting Started","doc":"Take a look at the `config/runtime.exs`. It includes a line like the following which reads from an environment variable named `SECRET`:\n\n    config :example, :secret, env!(\"SECRET\", :string!)\n\nIf you start your app using `iex -S mix` and enter into the `iex` shell:\n\n    iex> Application.get_env(:example, :secret)\n    \"my-secret-dev\"\n\nThis value came from the `envs/.dev.env` file, which declares the following:\n\n    SECRET=my-secret-dev\n\nNext, let's try running a test. Open up the `test/example_test.exs` file and edit it so we have single test like the following:\n\n    test \"reads variables specific to an env\" do\n        assert \"my-secret-dev\" == Application.get_env(:example, :secret)\n    end\n\nThen run `mix test`.  \n\nUh oh! The test fails because `Application.get_env(:example, :secret)` returned `\"my-secret-test\"` and not `\"my-secret-dev\"`.\n\nTake a look at the `envs/.test.env` file and see how the variable is declared there. We can now adjust our test so the assertion matches the value we declared in our `.test.env`:\n\n    test \"reads variables specific to an env\" do\n        assert \"my-secret-test\" == Application.get_env(:example, :secret)\n    end\n\nRunning `mix test` now passes!\n\n>","ref":"getting_started.html#environment-specific-env-files"},{"type":"extras","title":"Core Concept: variables are read from an environment-specific file {: .info} - Getting Started","doc":">\n> Just like with Elixir's regular config files, `Dotenvy` loads the appropriate\n> env file depending on your environment. Look at how the `config_env()` function\n> is used in `runtime.exs` to determine the file name; different values are\n> declared in the `.test.env` and `.dev.env`.","ref":"getting_started.html#core-concept-variables-are-read-from-an-environment-specific-file-info"},{"type":"extras","title":"Environment Variables - Getting Started","doc":"Next, let's try to access the environment variable directly:\n\n    iex> System.get_env(\"SECRET\")\n    nil\n\nWhat happened? `Application.get_env(:example, :secret)` worked, so why doesn't `System.get_env(\"SECRET\")` see the variable?\n\nThe answer to this riddle is that `Dotenvy` is read-only: `Dotenvy` does not _set_ environment variables. This helps keep things locked down. It may be counter-intuitive, but `Dotenvy` doesn't even necessarily read environment variables!  _`Dotenvy` only reads the inputs you give it_. `Dotenvy` only reads environment variables if you pass it the output from `System.get_env()`.\n\n>","ref":"getting_started.html#environment-variables"},{"type":"extras","title":"Core Concept: `Dotenvy` does not **set** ENV vars {: .info} - Getting Started","doc":">\n> Any variables you declare in your from in your `env` files are _not_ exported\n> back to the system; i.e. `System.put_env/2` is NOT called. In other words,\n> declaring a variable `FOO` in one of your parsed `.env` files does not mean\n> you can use `System.get_env/2` to retrieve it later. This encapsulation is by design!\n> If you want to set environment variables, you must do it explicitly.","ref":"getting_started.html#core-concept-dotenvy-does-not-set-env-vars-info"},{"type":"extras","title":"Establishing a contract - Getting Started","doc":"One of the tenets of the [12-factor App](https://12factor.net/) is to have a _clean contract_ with the underlying operating system, offering maximum portability between execution environments. Our `runtime.exs` is largely responsible for this: it dictates exactly which variables it needs.\n\nTo see this in action, let's add another configuration setting to our app by adding the following line to our example `runtime.exs`:\n\n    config :example, :password, env!(\"PASSWORD\", :string!)\n\nThen stop and restart your app by pressing ctrl-C and running `iex -S mix` once more.  You will see an error:\n\n    ** (RuntimeError) Environment variable PASSWORD not set\n\nThe application is declaring its contract by specifying that certain environment variables _must_ be present. Because the `PASSWORD` variable is not set, the application will not start because the contract has not been met.\n\nWe can provide the variable on the command line:\n\n        PASSWORD=xxxx iex -S mix\n\nAnd the app will start normally.\n\nAlternatively, you can provide this value in your env files. Add the following to your `envs/.dev.exs` file:\n\n        PASSWORD=xxxx\n\nYour application will now start in the dev environment. However if you try to run tests, you will once again see the `RuntimeError`. You can rectify this by supplying a value in the `envs/.test.dev` file.\n\nA good convention here is to have a default `.env` file loaded first which lists _all_ the variables that your app needs. That's a great place to put some documentation too!\n\n>","ref":"getting_started.html#establishing-a-contract"},{"type":"extras","title":"Core Concept: your app should dictate which variables it needs {: .info} - Getting Started","doc":">\n> if your application _needs_ certain configuration values to run, then it should\n> _demand_ that those values are set.  The implication is that if those values aren't\n> there, there's no point in starting the app because it can't do what it needs to do.\n> This is the _contract with the environment_.","ref":"getting_started.html#core-concept-your-app-should-dictate-which-variables-it-needs-info"},{"type":"extras","title":"Type-casting - Getting Started","doc":"All environment variables store string values. `Dotenvy.env!/2` and `Dotenvy.env!/3` have as their second argument an atom which determines how to convert the string value. For example, you may need to convert a `PORT` variable into an integer, other values may need to be booleans, and others may need to be atoms or modules.\n\nThere is some subtlety involved here when it comes to how empty values should be handled.\n\nLet's revisit our `PASSWORD` variable from the previous section. Let's try setting the environment variable to an empty value before we start our app:\n\n    $ PASSWORD= iex -S mix\n    ** (RuntimeError) Error converting variable PASSWORD to string!: non-empty value required\n\nUh-oh! This causes another error, this time not because the variable wasn't set (it was), but because its value was _empty_.\n\nLet's modify the line in our `runtime.exs` and replace `:string!` with `:string` (i.e. remove the exclamation point):\n\n    config :example, :password, env!(\"PASSWORD\", :string)\n\nNow the application starts fine, even if the `PASSWORD` value is empty. Probably a password needs to have a value, so using `:string!` for the second argument is probably more appropriate, but you can decide this on a case-by-case basis.\n\nUnderstanding type-casting is another core concept in helping to leverage `Dotenvy` so  your app get what it needs to run.\n\n>","ref":"getting_started.html#type-casting"},{"type":"extras","title":"Core Concept: type-casting {: .info} - Getting Started","doc":">\n> For each variable you read via `Dotenvy.env!/2` in `config/runtime.exs`, you\n> should consider what the resulting Elixir value needs to be. Can the value be empty? Are\n> `nil` values allowed? Choose the [conversion type](`t:Dotenvy.Transformer.conversion_type/0`) \n> that best supplies your app with the value it needs.\n\nSee the section on [releases](docs/guides/releases.md) for further information on how `Dotenvy` works in the context of a Mix release.","ref":"getting_started.html#core-concept-type-casting-info"},{"type":"extras","title":"Releases","doc":"# Releases\n\nOne of the hurdles when dealing with Elixir releases is that only certain files are packaged into them.  Any new ad-hoc files like our `.env` files are not included by default.  One way to ensure that our additional files get packaged into the release is to specify the [`:overlays` option](https://hexdocs.pm/mix/Mix.Tasks.Release.html#module-options) in your `mix.exs`. To do this we edit the `mix.exs` file to specify the `envs/` directory which contains your `.env` files:\n\n```elixir\n# mix.exs\n\ndef project do\n  [\n    app: :your_app,\n    # ... \n    releases: releases()\n  ]\nend\n\ndefp releases do\n  [\n    your_app: [\n        overlays: [\"envs/\"]\n    ]\n  ]\nend\n```\n\n>","ref":"releases.html"},{"type":"extras","title":"Core Concept: Overlays {: .info} - Releases","doc":">\n> When you specify a folder in the `overlays` option in your `mix.exs`, then the\n> _contents_ (and not the folder itself) will be copied to the root of the release.\n\nDuring day-to-day development, your `.env` files live inside the `envs/` folder, but when a release is built, they get copied to the _root_ of the release, so we cannot rely on relative paths in `runtime.exs`!  In our examples, we rely on `Path.expand/2`, `Path.absname/2`, and the presence of the `RELEASE_ROOT` system environment variable to resolve our relative paths into absolute paths.\n\n```elixir\nenv_dir_prefix = System.get_env(\"RELEASE_ROOT\") || Path.expand(\"./envs/\")\n```\n\nThis is a simple trick to ensure that we always have a fully-qualified path to where our `.env` files live. This pattern is repeated throughout the documentation because it is so important! \n\nPutting your `.env` files inside a folder named `envs/` is merely a convention: you are free to store them where you wish, but keep in mind that it is easier to deal with _folders_ than it is with individual files.  See the documentation on [Mix Release Overlays](https://hexdocs.pm/mix/Mix.Tasks.Release.html#module-overlays) for more information.\n\nOur `config/runtime.exs` will look something like the following. Note that the folder referenced in the `mix.exs` overlays section (`envs/`) must correspond with the path referenced in `config/runtime.exs`.\n\n```elixir\nimport Config\nimport Dotenvy\n\n# For local development, read dotenv files inside the envs/ dir;\n# for releases, read them at the RELEASE_ROOT\nenv_dir_prefix = System.get_env(\"RELEASE_ROOT\") || Path.expand(\"./envs/\")\n\nsource!([\n    Path.absname(\".env\", env_dir_prefix),\n    Path.absname(\".#{config_env()}.env\", env_dir_prefix),\n    Path.absname(\".#{config_env()}.overrides.env\", env_dir_prefix),\n    System.get_env()\n  ])\n```\n\nRemember that is always safer to use an absolute path. This is especially important when working with umbrella apps or [Livebooks](guides/livebooks.md)!","ref":"releases.html#core-concept-overlays-info"},{"type":"extras","title":"Umbrella Apps - Releases","doc":"Elixir [Umbrella Projects](https://elixir-lang.org/getting-started/mix-otp/dependencies-and-umbrella-projects.html) consume configuration slightly differently due to how they are organized.\n\nIn particular, you have to be very careful about relative paths when working in an umbrella project. Depending on what you're doing, the path may be _relative to a single application_ instead of relative to the root of the repository. As elsewhere, using `Path.expand/1` and `Path.absname/2` is a good way to anchor your `config/runtime.exs` so it resolves to the same directory no matter if the app is running locally or as a release.\n\nOnce again, the winning pattern for your `config/runtime.exs` will look something like this:\n\n```elixir\nenv_dir_prefix = System.get_env(\"RELEASE_ROOT\") || Path.expand(\"./envs/\")\n\nsource!([\n    Path.absname(\".env\", env_dir_prefix),\n    Path.absname(\".#{config_env()}.env\", env_dir_prefix),\n    Path.absname(\".#{config_env()}.overrides.env\", env_dir_prefix),\n    System.get_env()\n  ])\n```","ref":"releases.html#umbrella-apps"},{"type":"extras","title":"Changing the envs/ folder - Releases","doc":"What if you wish to keep your `.env` files in some other folder?  No problem. You just need to update your `runtime.exs` and your `mix.exs` so the `:overlays` option corresponds to the folder name.\n\nFor example, here's what you would do if wanted to keep your `.env` files inside a directory named `xyz`:\n\n```elixir\n# config/runtime.exs\nenv_dir_prefix = System.get_env(\"RELEASE_ROOT\") || Path.expand(\"./xyz/\")\n# ... etc...\n```\n\n```elixir\n# mix.exs\n  defp releases do\n    [\n      my_app: [\n        overlays: [\"xyz/\"]\n      ]\n    ]\n  end\n```\n\nSome languages/frameworks store `.env` files at the root of the application, but this isn't easily compatible with Elixir releases.  Rather than trying to push the river, we recommend choosing a sub-folder and leveraging the `:overlays` option.","ref":"releases.html#changing-the-envs-folder"},{"type":"extras","title":"Phoenix","doc":"# Phoenix\n\nThis page shows you how to either generate a new Phoenix application using the [`dotenvy_generators` package](docs/reference/generators.md) OR how to retrofit the config files of an existing Phoenix app.","ref":"phoenix.html"},{"type":"extras","title":"Creating a new Phoenix application that uses Dotenvy - Phoenix","doc":"Make sure you have installed the [`dotenvy_generators`](docs/reference/generators.md) before continuing!\n\nIn a new terminal window, you can run the new task to generate a new Phoenix app, e.g. `mix phx.new.dotenvy hello`.  This should generate a functional Phoenix application that leverages `Dotenvy` for its configuration.\n\nHave a look over the file structure: notice the `envs/` directory.  The files there house the values read at _runtime_, whereas the various config files inside of `config/` have been cleaned up so they focus on providing settings that must be defined at compile-time.","ref":"phoenix.html#creating-a-new-phoenix-application-that-uses-dotenvy"},{"type":"extras","title":"Manually Editing Files - Phoenix","doc":"If you have an existing Phoenix application and you want to modify it to use `Dotenvy`, then you can reference the files below as a guideline for editing your configuration files.\n\nPay attention to how the files are organized: most of the configuration has been moved into the `runtime.exs` leaving only minimal bits in the env-specific compile-time configs. Remember that one of the guiding principles of `Dotenvy` is to use runtime configuration whenever possible.\n\n>","ref":"phoenix.html#manually-editing-files"},{"type":"extras","title":"Replace the values to match your app {: .warning} - Phoenix","doc":">\n> You will need to replace `YourApp`, `YourAppWeb`, and `:your_app` with\n> the appropriate modules and app name for _your application_. Don't just copy\n> and paste these sample files!  And make sure you don't overwrite any existing config\n> for other apps/services that might not be present in this example.\n\n### `config/config.exs`\n\n```elixir\n# config/config.exs\n# This file is responsible for configuring your application\n# and its dependencies with the aid of the Config module.\n#\n# This configuration file is loaded before any dependency and\n# is restricted to this project.\n\n# General application configuration\nimport Config\n\nconfig :your_app,\n  ecto_repos: [YourApp.Repo],\n  generators: [timestamp_type: :utc_datetime]\n\n# Configures the endpoint\nconfig :your_app, YourAppWeb.Endpoint,\n  adapter: Bandit.PhoenixAdapter,\n  render_errors: [\n    formats: [html: YourAppWeb.ErrorHTML, json: YourAppWeb.ErrorJSON],\n    layout: false\n  ],\n  pubsub_server: YourApp.PubSub,\n  live_view: [signing_salt: \"xyz123AB\"]\n\n# Configure esbuild (the version is required)\nconfig :esbuild,\n  version: \"0.17.11\",\n  your_app: [\n    args:\n      ~w(js/app.js --bundle --target=es2017 --outdir=../priv/static/assets --external:/fonts/* --external:/images/*),\n    cd: Path.expand(\"../assets\", __DIR__),\n    env: %{\"NODE_PATH\" => Path.expand(\"../deps\", __DIR__)}\n  ]\n\n# Configure tailwind (the version is required)\nconfig :tailwind,\n  version: \"3.4.3\",\n  your_app: [\n    args: ~w(\n      --config=tailwind.config.js\n      --input=css/app.css\n      --output=../priv/static/assets/app.css\n    ),\n    cd: Path.expand(\"../assets\", __DIR__)\n  ]\n\n# Configures Elixir's Logger (compile-time config)\nconfig :logger, :console,\n  format: \"$time $metadata[$level] $message\\n\",\n  metadata: [:request_id]\n\n# Import environment specific config. This must remain at the bottom\n# of this file so it overrides the configuration defined above.\nimport_config \"#{config_env()}.exs\"\n```\n\n### `config/dev.exs`\n\n```elixir\n# config/dev.exs\nimport Config\n\n# Compile-time configuration includes code_reloader, debug_errors, and force_ssl\n# https://hexdocs.pm/phoenix/Phoenix.Endpoint.html#module-compile-time-configuration\nconfig :your_app, YourAppWeb.Endpoint,\n  code_reloader: true,\n  debug_errors: true,\n  force_ssl: false,\n  # Watch static and templates for browser reloading.\n  live_reload: [\n    patterns: [\n      ~r\"priv/static/(?!uploads/).*(js|css|png|jpeg|jpg|gif|svg)$\",\n      ~r\"priv/gettext/.*(po)$\",\n      ~r\"lib/your_app_web/(controllers|live|components)/.*(ex|heex)$\"\n    ]\n  ],\n  # The watchers configuration can be used to run external\n  # watchers to your application. For example, we can use it\n  # to bundle .js and .css sources.\n  # Watchers can be configured at runtime but are unlikely to change\n  watchers: [\n    esbuild: {Esbuild, :install_and_run, [:your_app, ~w(--sourcemap=inline --watch)]},\n    tailwind: {Tailwind, :install_and_run, [:your_app, ~w(--watch)]}\n  ]\n\n# Compile-time config\nconfig :phoenix_live_view,\n  debug_heex_annotations: true,\n  enable_expensive_runtime_checks: true\n\n# Enable dev routes for dashboard and mailbox (compile-time config)\nconfig :your_app, dev_routes: true\n\n# Do not include metadata nor timestamps in development logs\nconfig :logger, :console, format: \"[$level] $message\\n\"\n```\n\n### `config/test.exs`\n\n```elixir\n# config/test.exs\nimport Config\n\n# Print only warnings and errors during test\nconfig :logger, level: :warning\n```\n\n### `config/prod.exs`\n\n```elixir\nimport Config\n\n# Compile-time configuration includes code_reloader, debug_errors, and force_ssl\n# https://hexdocs.pm/phoenix/Phoenix.Endpoint.html#module-compile-time-configuration\nconfig :your_app, YourAppWeb.Endpoint,\n  code_reloader: false,\n  debug_errors: false,\n  force_ssl: false\n\n# Do not print debug messages in production\nconfig :logger, level: :info\n```\n\n### `config/runtime.exs`\n\n```elixir\n# config/runtime.exs\nimport Config\nimport Dotenvy\n\n# For local development, read dotenv files inside the envs/ dir;\n# for releases, read them at the RELEASE_ROOT\nenv_dir_prefix = System.get_env(\"RELEASE_ROOT\") || Path.expand(\"./envs/\")\n\nsource!(\n  [\n    Path.absname(\".env\", env_dir_prefix),\n    Path.absname(\".#{config_env()}.env\", env_dir_prefix),\n    Path.absname(\".#{config_env()}.overrides.env\", env_dir_prefix),\n    System.get_env()\n  ],\n  require_files: [Path.absname(\".env\", env_dir_prefix)]\n)\n\n\n# config/runtime.exs is executed for all environments, including\n# during releases. It is executed after compilation and before the\n# system starts. Do not define any compile-time configuration in here,\n# as it won't be applied.\n\n#","ref":"phoenix.html#replace-the-values-to-match-your-app-warning"},{"type":"extras","title":"Using releases - Phoenix","doc":"#\n# If you use `mix release`, you need to explicitly enable the server\n# by passing the PHX_SERVER=true when you start it:\n#\n#     PHX_SERVER=true bin/your_app start\n#\n# Alternatively, you can use `mix phx.gen.release` to generate a `bin/server`\n# script that automatically sets the env var above.\nif env!(\"PHX_SERVER\", :boolean!) do\n  config :your_app, YourAppWeb.Endpoint, server: true\nend\n\n# Initialize plugs at runtime for faster development compilation\n# values can be :runtime or :compile; must be :compile in prod (the default)\nconfig :phoenix, :plug_init_mode, env!(\"PHX_PLUGIN_INIT_MODE\", :existing_atom!)\n\n# Use Jason for JSON parsing in Phoenix\nconfig :phoenix, :json_library, Jason\n\nip =\n  env!(\"HTTP_INTERFACE\", fn val ->\n    with [_] <- String.split(val, \".\"),\n         [_] <- String.split(val, \":\") do\n      raise \"Invalid IP address specified\"\n    else\n      parts -> parts |> Enum.map(&String.to_integer/1) |> List.to_tuple()\n    end\n  end)\n\necto_socket_options = if env!(\"ECTO_IPV6\", :boolean!), do: [:inet6], else: []\n\nconfig :your_app, YourApp.Repo,\n  # ssl: true,\n  url: env!(\"DATABASE_URL\", :string!),\n  pool: env!(\"PG_POOL\", :module?),\n  pool_size: env!(\"POOL_SIZE\", :integer!),\n  socket_options: ecto_socket_options,\n  stacktrace: env!(\"ECTO_STACKTRACE\", :boolean),\n  show_sensitive_data_on_connection_error:\n    env!(\"SHOW_SENSITIVE_DATA_ON_CONNECTION_ERROR\", :boolean)\n\nif env!(\"ENABLE_DISTRIBUTED_MODE\", :boolean) do\n  config :your_app, :dns_cluster_query, env!(\"DNS_CLUSTER_QUERY\", :string)\nend\n\nconfig :your_app, YourAppWeb.Endpoint,\n  cache_static_manifest: env!(\"PHX_CACHE_STATIC_MANIFEST\", :string?),\n  check_origin: env!(\"HTTP_CHECK_ORIGIN\", :boolean),\n  http: [ip: ip, port: env!(\"PORT\", :integer!)],\n  secret_key_base: env!(\"SECRET_KEY_BASE\", :string!),\n  # Used to build URLs\n  url: [\n    host: env!(\"PHX_HOST\", :string!),\n    port: env!(\"PHX_URL_PORT\", :integer!),\n    scheme: env!(\"PHX_URL_SCHEME\", :string!)\n  ]\n\n#","ref":"phoenix.html#using-releases"},{"type":"extras","title":"SSL Support - Phoenix","doc":"#\n# To get SSL working, you will need to add the `https` key\n# to your endpoint configuration:\n#\n#     config :your_app, YourAppWeb.Endpoint,\n#       https: [\n#         ...,\n#         port: 443,\n#         cipher_suite: :strong,\n#         keyfile: System.get_env(\"SOME_APP_SSL_KEY_PATH\"),\n#         certfile: System.get_env(\"SOME_APP_SSL_CERT_PATH\")\n#       ]\n#\n# The `cipher_suite` is set to `:strong` to support only the\n# latest and more secure SSL ciphers. This means old browsers\n# and clients may not be supported. You can set it to\n# `:compatible` for wider support.\n#\n# `:keyfile` and `:certfile` expect an absolute path to the key\n# and cert in disk or a relative path inside priv, for example\n# \"priv/ssl/server.key\". For all supported SSL configuration\n# options, see https://hexdocs.pm/plug/Plug.SSL.html#configure/1\n#\n# We also recommend setting `force_ssl` in your config/prod.exs,\n# ensuring no data is ever sent via http, always redirecting to https:\n#\n#     config :your_app, YourAppWeb.Endpoint,\n#       force_ssl: [hsts: true]\n#\n# Check `Plug.SSL` for all available options in `force_ssl`.\n#\n# In order to use HTTPS in development, a self-signed\n# certificate can be generated by running the following\n# Mix task:\n#\n#     mix phx.gen.cert\n#\n# Run `mix help phx.gen.cert` for more information.\n#\n# The `http:` config above can be replaced with:\n#\n#     https: [\n#       port: 4001,\n#       cipher_suite: :strong,\n#       keyfile: \"priv/cert/selfsigned_key.pem\",\n#       certfile: \"priv/cert/selfsigned.pem\"\n#     ],\n#\n# If desired, both `http:` and `https:` keys can be\n# configured to run both http and https servers on\n# different ports.\n\n#","ref":"phoenix.html#ssl-support"},{"type":"extras","title":"Configuring the mailer - Phoenix","doc":"#\n# In production you need to configure the mailer to use a different adapter.\n# Also, you may need to configure the Swoosh API client of your choice if you\n# are not using SMTP. Here is an example of the configuration:\n#\n#     config :your_app, YourApp.Mailer,\n#       adapter: Swoosh.Adapters.Mailgun,\n#       api_key: System.get_env(\"MAILGUN_API_KEY\"),\n#       domain: System.get_env(\"MAILGUN_DOMAIN\")\n#\n# For this example you need include a HTTP client required by Swoosh API client.\n# Swoosh supports Hackney and Finch out of the box:\n#\n#     config :swoosh, :api_client, Swoosh.ApiClient.Hackney\n#\n# See https://hexdocs.pm/swoosh/Swoosh.html#module-installation for details.\n# Configures Swoosh API Client\nif env!(\"SWOOSH_API_CLIENT\", :boolean) do\n  config :swoosh, api_client: env!(\"SWOOSH_API_CLIENT\", :module!), finch_name: YourApp.Finch\nelse\n  config :swoosh, api_client: false\nend\n\nconfig :your_app, YourApp.Mailer, adapter: env!(\"SWOOSH_MAILER_ADAPTER\", :module)\nconfig :swoosh, api_client: Swoosh.ApiClient.Finch, finch_name: YourApp.Finch\nconfig :swoosh, local: env!(\"SWOOSH_LOCAL_MEMORY_STORAGE\", :boolean)\n```\n\n### `envs/.env`\n\nThe shared/default values can be listed and documented here.  The `.dev.env` and other `.env` files can either copy this file in its entirety and modify the values (for easier diff comparisons), or they can include only the variables that they need to change.\n\n```env\n# envs/.env (default/shared config)\n# ###################\n# Distributed Node  #\n# ###################\n# If distributed mode is enabled, the DNS_CLUSTER_QUERY variable must have a value\nENABLE_DISTRIBUTED_MODE=false\nDNS_CLUSTER_QUERY=\n\n# #####################\n# Phoenix / Webserver #\n# #####################\n# The interface(s) to listen on.\n# Can be specified as the following formats:\n# `1.2.3.4` for IPv4 addresses (using period separators)\n# `1:2:3:4:5:6:7:8` for IPv6 addresses (using colon separators)\n# \n# To bind to loopback IPv4 address & prevent access from other machines: `127.0.0.1`\n# To allow access from other machines: `0.0.0.0`\n# \n# To enable IPv6 and bind on all interfaces: `0:0:0:0:0:0:0:0`\n# For local network only access: `0:0:0:0:0:0:0:1`\n# \n# See the documentation on https://hexdocs.pm/bandit/Bandit.html#t:options/0\n# for details about using IPv6 vs IPv4 and loopback vs public addresses.\nHTTP_INTERFACE=127.0.0.1\nHTTP_CHECK_ORIGIN=false\n# Port where HTTP requests will be accepted\nPORT=4000\n# A secret key used as a base to generate secrets for encrypting and signing data. \n# For example, cookies and tokens are signed by default, but they may also be \n# encrypted if desired. Must be set per application.\nSECRET_KEY_BASE=\n# Boolean indicating whether to start the Phoenix webserver implicitly.\n# Usually this is false for dev and true for prod\nPHX_SERVER=false\n# PHX_HOST, PHX_URL_PORT, and PHX_URL_SCHEME are used to create links\nPHX_HOST=localhost\n# Duplicate the ${PORT} value for http; 443 recommended for https\nPHX_URL_PORT=\n# scheme may be either http or https\nPHX_URL_SCHEME=http\n# Specifies the path to a cache manifes containing the digested version of \n# static files. This manifest is generated by the `mix assets.deploy` task,\n# which you should run after static files are built and\n# before starting your production server.\n# Leave empty for dev or when you do not need a manifest\nPHX_CACHE_STATIC_MANIFEST=\n# Set a higher stacktrace during development, e.g. 20, or set to false to disable.\n# Disabling is recommended in prod as building large stacktraces may be expensive.\nPHX_STACKTRACE_DEPTH=20\n# Initialize plugs at runtime for faster development compilation\n# values can be :runtime or :compile; must be :compile in prod (the default)\nPHX_PLUGIN_INIT_MODE=compile\n\n############\n# Database #\n############\n# The MIX_TEST_PARTITION environment variable can be used\n# to provide built-in test partitioning in CI environment.\n# Run `mix help test` for more information.\nMIX_TEST_PARTITION=\nPG_USERNAME=postgres\nPG_PASSWORD=postgres\nPG_HOSTNAME=localhost\nPG_DATABASE=your_app_dev\nPG_POOL=DBConnection.ConnectionPool\nPOOL_SIZE=10\n# DATABASE_URL format is `ecto://USER:PASS@HOST/DATABASE`\nDATABASE_URL=ecto://${PG_USERNAME}:${PG_PASSWORD}@${PG_HOSTNAME}/${PG_DATABASE}\nSHOW_SENSITIVE_DATA_ON_CONNECTION_ERROR=true\nECTO_IPV6=false\nECTO_STACKTRACE=true\n\n###################\n# Swoosh / Mailer #\n###################\n# Specify a module as a Swoosh API client for production adapters or set to false\n# to disable. See https://hexdocs.pm/swoosh/Swoosh.ApiClient.html\nSWOOSH_API_CLIENT=false\nSWOOSH_LOCAL_MEMORY_STORAGE=true\n# By default it uses the Swoosh.Adapters.Local adapter which stores the emails\n# locally. You can see the emails in your browser, at \"/dev/mailbox\".\n# For production it's recommended to configure a different adapter\nSWOOSH_MAILER_ADAPTER=Swoosh.Adapters.Local\n```\n\n### `envs/.dev.env`\n\n```env\n# envs/.dev.env\nHTTP_INTERFACE=127.0.0.1\nHTTP_CHECK_ORIGIN=false\nPORT=4000\nSECRET_KEY_BASE=6D1kPrXseBg9F1O9cZUd8ocIH7l3OgT9ZlopIXqDr+jYjNrcvbjqwNvzHPKIakcF\nPHX_URL_PORT=${PORT}\nPHX_PLUGIN_INIT_MODE=runtime\n\nPG_USERNAME=postgres\nPG_PASSWORD=postgres\nPG_HOSTNAME=localhost\nPG_DATABASE=your_app_dev\nPG_POOL=DBConnection.ConnectionPool\nPOOL_SIZE=10\n# DATABASE_URL format is `ecto://USER:PASS@HOST/DATABASE`\nDATABASE_URL=ecto://${PG_USERNAME}:${PG_PASSWORD}@${PG_HOSTNAME}/${PG_DATABASE}\nSHOW_SENSITIVE_DATA_ON_CONNECTION_ERROR=true\nECTO_IPV6=false\nECTO_STACKTRACE=true\n```","ref":"phoenix.html#configuring-the-mailer"},{"type":"extras","title":"envs/.test.env - Phoenix","doc":"```env\n# envs/.test.env\nHTTP_INTERFACE=127.0.0.1\nHTTP_CHECK_ORIGIN=false\nPORT=4002\nSECRET_KEY_BASE=514niO7SLGRKi01EN1fQUqekoyfHSQL0640m64tAkJSPG7anLA6iPxWrcIUzCgyA\nPHX_URL_PORT=${PORT}\nPHX_PLUGIN_INIT_MODE=runtime\n\nPG_USERNAME=postgres\nPG_PASSWORD=postgres\nPG_HOSTNAME=localhost\nPG_DATABASE=your_app_test\nPG_POOL=Ecto.Adapters.SQL.Sandbox\nPOOL_SIZE=10\n# DATABASE_URL format is `ecto://USER:PASS@HOST/DATABASE`\nDATABASE_URL=ecto://${PG_USERNAME}:${PG_PASSWORD}@${PG_HOSTNAME}/${PG_DATABASE}\nSHOW_SENSITIVE_DATA_ON_CONNECTION_ERROR=true\nECTO_IPV6=false\nECTO_STACKTRACE=true\n\nSWOOSH_MAILER_ADAPTER=Swoosh.Adapters.Test\n```\n\n### `envs/.prod.env`\n\nIn prod, you may have certain env variables provided by your host.  For example. [Fly.io](https://fly.io/) will define a number of env variables for you. It can be helpful to list them in your `.prod.env` file as a reminder.\n\n```env\n# envs/.prod.env\nENABLE_DISTRIBUTED_MODE=true\n# DNS_CLUSTER_QUERY= # set by Fly.io\n\nHTTP_INTERFACE=0:0:0:0:0:0:0:0\nHTTP_CHECK_ORIGIN=true\n# PORT= # set by Fly.io\n# SECRET_KEY_BASE= # Set by Fly.io\nPHX_SERVER=true\nPHX_HOST=your-app-website.com\nPHX_URL_PORT=443\nPHX_URL_SCHEME=https\nPHX_CACHE_STATIC_MANIFEST=priv/static/cache_manifest.json\nPHX_STACKTRACE_DEPTH=false\nPHX_PLUGIN_INIT_MODE=compile\n\n# DATABASE_URL= # set by Fly.io\nSHOW_SENSITIVE_DATA_ON_CONNECTION_ERROR=false\nECTO_IPV6=true\nECTO_STACKTRACE=false\n\nSWOOSH_API_CLIENT=Swoosh.ApiClient.Finch\nSWOOSH_LOCAL_MEMORY_STORAGE=false\n# For production it's recommended to configure a different adapter...\n# SWOOSH_MAILER_ADAPTER=Swoosh.Adapters.Local\n```","ref":"phoenix.html#envs-test-env"},{"type":"extras","title":"Minimal Setup","doc":"# Minimal Setup\n\nHere is another demonstration of how you can configure your app for maximum simplicity: less is more.  Per the [12-Factor App](https://12factor.net/), this strategy helps \"Minimize divergence between development and production\". In some cases, the `dev`, `prod`, and `test` versions of the app can be _identical_ (even down the the md5 hash) because the only differences are the configuration values supplied at runtime.\n\nTo see this strategy in action, we will abolish the `dev.ex`, `prod.exs`, and `test.exs` _entirely_ and leave _only_ the compile-time config (`config/config.exs`) and the runtime configuration (`config/runtime.exs`).\n\nTo try this out:\n\n1. Use the `dot.new` generator to generate a new application, e.g. `mix dot.new sparse`\n2. Delete the env-specific compile-time configs: `config/dev.exs`, `config/prod.exs`, and `config/test.exs`.\n3. Remove the line at the end of `config/config.exs` that uses `import_config` to load these other files.  \n4. If environment-specific compile-time configurations are needed, update your`config/config.exs` to include blocks for each config environment.\n\nHere is an example of the stripped down `config/config.exs` file:\n\n### `config/config.exs`\n\n```elixir\n# compile-time config\nimport Config\n\n# Dev\nif config_env() == :dev do\n    config :logger, :console, \n        level: :debug,\n        format: \"[$level] $message\\n\"\nend\n\n# Test\nif config_env() == :test do\n    config :logger, :console, \n            level: :warning,\n            format: \"[$level] $message\\n\"\nend\n\n# Prod\nif config_env() == :prod do\n    config :logger, :console,\n        format: \"$time $metadata[$level] $message\\n\",\n        level: :info,\n        metadata: [:request_id]\nend\n```\n\n>","ref":"minimal.html"},{"type":"extras","title":"Runtime vs. Compile-time Considerations {: .info} - Minimal Setup","doc":">\n> Logger configuration is an interesting example -- because the `Logger` relies on\n> macros, it is affected by compile-time considerations. Although you _can_ configure\n> the logger level at runtime, that has a different effect than configuring it at\n> compile-time.  If you set the logger level to `:info` at runtime, then you will only\n> see info, warning, or error messages in the logs... but the important thing is that\n> _the calls to `Logger.debug/2` are still there in the code_. Calls to those functions\n> are still made; only the output is silenced.\n>\n> By comparison, if you set the logger level to `:info` in the compile-time config, all\n> calls to `Logger.debug/2` are _removed_ from the compiled application.  Calls are no\n> longer made to `Logger.debug/2` because _that function no longer exists_. That\n> distinction may not matter for small apps, but you can imagine that those\n> milliseconds can add up for mission-critical applications where performance is\n> paramount.\n>\n> TL;DR: Sometimes it is better to have the flexibility to change the logging level at\n> runtime (e.g. to help debug some tricky problem) and sometimes it's better to control\n> this at compile-time. You decide.\n\nPhoenix relies on macros to generate routes, so you must configure certain things at compile time: its routes are compiled into existence. Consider the following bits of configuration from a Phoenix application:\n\n```elixir\n# Compile-time config\nconfig :phoenix_live_view,\n  debug_heex_annotations: true,\n  enable_expensive_runtime_checks: true\n\n# Enable dev routes for dashboard and mailbox (compile-time config)\nconfig :your_app, dev_routes: true\n```\n\nThose _must_ be defined at compile-time because it controls how the application gets built.  You can put these types of configuration details into the appropriate environment block in the compile-time `config.exs`.\n\n### `config/runtime.exs`\n\nYour `runtime.exs` can include whatever it needs to, i.e. all the application configuration that can happen at runtime and you know that the calls to `Dotenv.source/2` will be responsible for loading up the proper `.env` files for the given environment.\n\n```elixir\nimport Config\nimport Dotenvy\n\nenv_dir_prefix = System.get_env(\"RELEASE_ROOT\") || Path.expand(\"./envs/\")\n\nsource!([\n    Path.absname(\".env\", env_dir_prefix),\n    Path.absname(\".#{config_env()}.env\", env_dir_prefix),\n    Path.absname(\".#{config_env()}.overrides.env\", env_dir_prefix),\n    System.get_env()\n  ])\n\n\nconfig :myapp, MyApp.Repo,\n  database: env!(\"DATABASE\", :string!),\n  username: env!(\"USERNAME\", :string),\n  password: env!(\"PASSWORD\", :string),\n  hostname: env!(\"HOSTNAME\", :string!),\n  pool_size: env!(\"POOL_SIZE\", :integer),\n  adapter: env(\"ADAPTER\", :module, Ecto.Adapters.Postgres),\n  pool: env!(\"POOL\", :module?)\n\n  # etc...\n```","ref":"minimal.html#runtime-vs-compile-time-considerations-info"},{"type":"extras","title":"Optimizing Compilation Time - Minimal Setup","doc":"By default, your Elixir application compiles differently between environments, so you end up with multiple compiled artifacts in your `_build` directory.  All this extra time spent compiling and re-compiling can really add up, so it's worth asking whether or not it's necessary.\n\nIf you have an app that can be _fully_ configured at runtime and there are no differences between the compiled versions, then you can set the `:build_per_environment` option in your`mix.exs` so that all environments use the same compiled code and you no longer need to re-compile your app between environments.\n\n```elixir\n  def project do\n    [\n      app: :my_app,\n      version: \"0.1.0\",\n      elixir: \"~> 1.18\",\n      start_permanent: Mix.env() == :prod,\n      deps: deps(),\n      releases: releases(),\n      build_per_environment: false\n    ]\n  end\n```\n\nIn some cases, you may be able to leverage this even if there _are_ differences between `dev` and `prod`, but you'll have to explore the subtleties with your particular app. For example, if `dev` and `test` can share the same compiled artifacts, that may never conflict with the `prod` artifacts which may only need to be compiled on a build machine as part of your deployment pipeline. YMMV.\n\nSee [Mix Project configuration](https://hexdocs.pm/mix/Mix.Project.html#module-configuration) for the official docs.","ref":"minimal.html#optimizing-compilation-time"},{"type":"extras","title":"See Also - Minimal Setup","doc":"See the [article](https://fireproofsocks.medium.com/configuration-in-elixir-with-dotenvy-8b20f227fc0e) about using `Dotenvy` in your application for more discussion on this and other approaches.","ref":"minimal.html#see-also"},{"type":"extras","title":"Livebooks","doc":"# Livebooks\n\nWhen running an Elixir application from a [Livebook](https://livebook.dev/), we run into a familiar problem of resolving relative file names. The `.env` path that made sense when you started your app from your application root using `mix` will not work when starting the app from a `Livebook` server executing in some faraway directory.\n\nIn order to overcome this problem, it is important to resolve relative paths into absolute paths.  As demonstrated elsewhere, this can be done using `Path.expand/1` and `Path.absname/2`:\n\n```elixir\nenv_dir_prefix = System.get_env(\"RELEASE_ROOT\") || Path.expand(\"./envs/\")\n\nsource!([\n    Path.absname(\".env\", env_dir_prefix),\n    Path.absname(\".#{config_env()}.env\", env_dir_prefix),\n    Path.absname(\".#{config_env()}.overrides.env\", env_dir_prefix),\n    System.get_env()\n  ])\n```\n\nNow your Livebooks can install apps using `Dotenvy` as you would other applications. In your Livebook setup, include a `Mix.install/2` block like the following:\n\n```elixir\nMix.install(\n  [\n    {:app_using_dotenvy, path: \"/path/to/app_using_dotenvy\", env: :dev}\n  ],\n  config_path: :app_using_dotenvy,\n  lockfile: :app_using_dotenvy\n)\n```","ref":"livebooks.html"},{"type":"extras","title":"Fly.io","doc":"# Fly.io\n\n`Dotenvy` can be used when deploying an app to [Fly.io](https://fly.io/); other hosting services may require customizations similar to the ones demonstrated here. Following the instructions on the [Phoenix](docs/phoenix.md) page will get you most of the way there, but you also need to modify the `Dockerfile` so it copies over your `envs/` directory: the `.env` files must be available to the `release` command. The default Fly.io Dockerfile needs a small alteration to ensure that your `.env` files are along for the ride.","ref":"flyio.html"},{"type":"extras","title":"Dockerfile - Fly.io","doc":"A `Dockerfile` gets generated when you setup your app (it is ultimately generated from `mix phx.gen.release --docker`). The `Dockerfile` is ultimately what is responsible for running `mix release`, so you need to ensure that all of your files and folders are copied into the container.\n\nIn the examples provided in this documentation, `.env` files are stored in a directory named `envs/`.  If that's the directory housing your `.env` files, then you will need to ensure that folder gets copied into the Docker container so it can properly build the release.  To do this, you will need to add a line `COPY envs envs` _before_ the `Run mix release` command.\n\n```docker\n# ... existing Docker stuff...\n\n# Compile the release\nRUN mix compile\n\n# Changes to config/runtime.exs don't require recompiling the code\nCOPY config/runtime.exs config/\n\nCOPY rel rel\n\n# <---- make the directory of envs available to the release!!!\nCOPY envs envs\n\nRUN mix release\n\n# ... existing Docker stuff cont'd...\n```\n\nOnce that line is there, then running `fly deploy` and other `flyctl` commands should be able to build the release and include the `.env` files as expected. You can test the standard GitHub workflows by pushing changes to `main`.","ref":"flyio.html#dockerfile"},{"type":"extras","title":"The importance of path names - Fly.io","doc":"Because your app executes inside a Docker container, it is really important to use `Path.absname/2` to reference your `.env` files. The reason we mention this _ad nauseam_ in these docs is because it's so critical.  Different release strategies may set the `RELEASE_ROOT` variable with or without a trailing slash, and that can mean that `Dotenvy` can't find your `.env` files.  It can be helpful to specify the `:require_files` option to `Dotenvy.source/2` just to help alert you when the paths may be pointing to the wrong place, e.g.\n\n```elixir\nenv_dir_prefix = System.get_env(\"RELEASE_ROOT\") || Path.expand(\"./envs/\")\n\nsource!(\n  [\n    Path.absname(\".env\", env_dir_prefix),\n    Path.absname(\".#{config_env()}.env\", env_dir_prefix),\n    Path.absname(\".#{config_env()}.overrides.env\", env_dir_prefix),\n    System.get_env()\n  ],\n  require_files: [Path.absname(\".env\", env_dir_prefix)]\n)\n```","ref":"flyio.html#the-importance-of-path-names"},{"type":"extras","title":"Environment Variables - Fly.io","doc":"Fly.io sets a handful of environment variables when it deploys an app (which you can inspect by running `System.get_env()` from an `iex` shell).  The following are the most significant that are used in the config files generated by the [Dotenvy Generators](https://hexdocs.pm/dotenvy_generators/):\n\n- `RELEASE_ROOT`\n- `DATABASE_URL`\n- `PHX_HOST`\n- `PORT`\n- `PHX_SERVER`\n- `DNS_CLUSTER_QUERY`\n- `SECRET_KEY_BASE`\n\nSee the [Phoenix](docs/phoenix.md) page for getting a Phoenix app to run using `Dotenvy`.","ref":"flyio.html#environment-variables"},{"type":"extras","title":"1Password","doc":"# 1Password\n\n[1Password](https://1password.com/) is a popular password manager. The [1Password CLI](https://developer.1password.com/docs/cli/get-started/) utility is a convenient way to read sensitive data out of a password vault.  The approach taken here is similar to what other password tools may require.\n\nThe syntax for accessing values out of a 1Password item is this:\n\n```sh\nop:// / /[section-name/] \n```\n\nAssuming you have the `op` command installed, you can execute it and other system commands in your `.env` files by using the `$()` syntax (available since version 1.0.0 of `Dotenvy`).  For example:\n\n```sh\nDB_PASSWORD=$(op read op://MyVault/FooDatabase/password);\nAPI_KEY=$(op read op://MyVault/ImportantAPI/key);\n```","ref":"1password.html"},{"type":"extras","title":"Executing Shell commands prior to version 1.0.0 - 1Password","doc":"If you need this functionality in versions of `Dotenvy` prior to version 1.0.0, you must do a bit more footwork and rely on `System.shell/2`.  Use this with caution!\n\nIf you're unable to upgrade to version 1.0.0 or the shell commands you are attempting to run are not somehow not supported by the `Dotenvy.Parser`, please file a bug! And then try the following.\n\nCreate a shell script that exports the necessary ENV vars, e.g. `secrets.sh`:\n\n```sh\n#!/bin/bash\nexport DB_PASSWORD=$(op read op://MyVault/FooDatabase/password);\nexport API_KEY=$(op read op://MyVault/ImportantAPI/key);\n```\n\nNext, we need to execute this file AND return all the values!  We do this by using `&&` in our shell command, e.g.\n\n```elixir\nSystem.shell(~s'bash -c \"source secrets.sh && env\"')\n```\n\nWhen it's all put together in your `config/runtime.exs` it should look something like this:\n\n```elixir\n# config/runtime.exs\nimport Config\nimport Dotenvy\n\nenv_dir_prefix = System.get_env(\"RELEASE_ROOT\") || Path.expand(\"./envs/\")\n\n{raw_envs, _} = System.shell(~s'bash -c \"source #{Path.absname(\"secrets.sh\", env_dir_prefix)} && env\"')\n{:ok, system_env_vars} = Dotenvy.Parser.parse(raw_envs)\n\nsource!([\n    Path.absname(\".env\", env_dir_prefix),\n    Path.absname(\".#{config_env()}.env\", env_dir_prefix),\n    Path.absname(\".#{config_env()}.overrides.env\", env_dir_prefix),\n    system_env_vars\n  ])\n```","ref":"1password.html#executing-shell-commands-prior-to-version-1-0-0"},{"type":"extras","title":"See Also - 1Password","doc":"You may need to make some adjustments to your security settings your your command line utility doesn't prompt you for authorization each time you run it:","ref":"1password.html#see-also"},{"type":"extras","title":"Philosophy","doc":"# Philosophy\n\n`Dotenvy` was born out of a desire to make Elixir's configuration more explicit and cleave more closely to how many other languages and frameworks are configured. Many Elixir apps contain a mishmash of compile-time and runtime configuration that can be confusing or unpredictable. Sometimes the choice of where to configure a particular thing is more the product of _convenience_ rather than the result of thoughtful deliberation. `Dotenvy` attempts to take the guesswork out of the configuration by encouraging us to be intentional and explicit with our app and the configuration values it needs.\n\nTaking its cue from the [12-factor app](https://12factor.net/), `Dotenvy`'s modus operandi is as follows:\n\n1. If something _can_ be configured at runtime, then it _should_ be configured at runtime. Keep the compile-time configuration for things that must be configured at compile-time.\n\n2. [Store configuration in the environment](https://12factor.net/config). Elixir apps configured with `Dotenvy` tend to be smaller and lighter because they are not as burdened with conditionals. Let the app be the app and let the environment provide the configuration it needs.\n\n3. Have a clean contract with the underlying operating system: the `config/runtime.exs` can leverage `Dotenvy.env!/2` to specify exactly the values it needs in a simple declarative way.\n\n4. Be suitable for deployment on modern cloud platforms: any system/software that can provide environment variables will work with `Dotenvy`.\n\n5. Minimize divergence between development and production: you can end up with a version of the app that is identical across all environments.","ref":"philosophy.html"},{"type":"extras","title":"Dotenv File Format","doc":"# Dotenv File Format\n\n`.env` files (a.k.a. \"dotenv\") store key-value pairs in a format descended from\nsimple bash files that exported environment variables.\n\nThis implementation cleaves closely to the format described by the original [dotenv](https://github.com/bkeepers/dotenv) package, but it is not a direct match (by design).\n\nTypically, a dotenv (`.env`) file is formatted into simple key-value pairs:\n\n    S3_BUCKET=YOURS3BUCKET\n    SECRET_KEY=YOURSECRETKEYGOESHERE\n\nOptionally, you may add `export` in front of each line so you can `source` the file in Bash:\n\n    export S3_BUCKET=YOURS3BUCKET\n    export SECRET_KEY=YOURSECRETKEYGOESHERE","ref":"dotenv-file-format.html"},{"type":"extras","title":"Variable Names - Dotenv File Format","doc":"For the sake of portability (and sanity), environment variable names must consist solely of letters, digits, and the underscore ( `_` ) and must not begin with a digit. In regex-speak, the names must match the following pattern:\n\n    [a-zA-Z_]+[a-zA-Z0-9_]*","ref":"dotenv-file-format.html#variable-names"},{"type":"extras","title":"Example variable names - Dotenv File Format","doc":"DATABASE_URL  # ok  \n    foobar        # ok  \n    NO-WORK       # <-- invalid !!!\n    ÜBER          # <-- invalid !!!\n    2MUCH         # <-- invalid !!!","ref":"dotenv-file-format.html#example-variable-names"},{"type":"extras","title":"Values - Dotenv File Format","doc":"Values are to the right of the equals sign. They may be quoted. Using single quotes will prevent variables from being interpolated and any command substitution will not be executed.\n\n    SIMPLE=xyz123\n    INTERPOLATED=\"Multiple\\nLines and variable substitution: ${SIMPLE}\"\n    NON_INTERPOLATED='raw text without variable interpolation'\n    MULTILINE = \"\"\"\n    long text here,\n    e.g. a private SSH key\n    \"\"\"","ref":"dotenv-file-format.html#values"},{"type":"extras","title":"Escape Sequences - Dotenv File Format","doc":"The following character strings will be interpreted (i.e. escaped) as specific codepoints in the same way you would expect if the values were assigned inside a script. Remember: when a text file is read, it is read as a series of utf8 encoded code points. Character sequences like `\\n` have no special meaning until they are \"escaped\" and a combination of codepoints is replaced by a single codepoint.\n\n- `\\n` Linefeed (aka newline); `<<92, 110>>` -> `<<10>>`\n- `\\r` Carriage return; `<<92, 114>>` -> `<<13>>`\n- `\\t` Tab; -> `<<92, 116>>` -> `<<9>>`\n- `\\f` Form feed; -> `<<92, 102>>` -> `<<12>>`\n- `\\b` Backspace; -> `<<92, 98>>` -> `<<8>>`\n- `\\\"` Double-quote; ->  `<<92, 34>>` -> `<<34>>`\n- `\\'` Single-quote; -> `<<92, 39>>` -> `<<39>>`\n- `\\\\` Backslash; -> `<<92, 92>>` -> `<<92>>`\n- `\\uFFFF` Unicode escape (4 hex characters to denote the codepoint)\n\nIf a backslash precedes any other character, the backslash will be ignored and removed from output.","ref":"dotenv-file-format.html#escape-sequences"},{"type":"extras","title":"Interpolation (a.k.a. Variable Substitution) - Dotenv File Format","doc":"Values left unquoted or wrapped in double-quotes will interpolate variables in the `${VAR}` syntax. This can be useful for referencing existing system environment variables or to reference variables previously parsed.\n\nFor example:\n\n    USER=admin\n    EMAIL=${USER}@example.org\n    DATABASE_URL=\"postgres://${USER}@localhost/my_database\"\n    CACHE_DIR=${PWD}/cache\n\nMulti-line values (e.g. private keys) can use the triple-quoted heredoc syntax:\n\n    PRIVATE_KEY=\"\"\"\n    -----BEGIN RSA PRIVATE KEY-----\n    ...\n    HkVN9...\n    ...\n    -----END DSA PRIVATE KEY-----\n    \"\"\"","ref":"dotenv-file-format.html#interpolation-a-k-a-variable-substitution"},{"type":"extras","title":"Non-Interpolated - Dotenv File Format","doc":"If your values must retain `${}` in their output, wrap the value in single quotes, e.g.:\n\n    PASSWORD='!@G0${k}k'\n    MESSAGE_TEMPLATE='''\n        Hello ${PERSON},\n\n        Nice to meet you!\n    '''","ref":"dotenv-file-format.html#non-interpolated"},{"type":"extras","title":"Comments - Dotenv File Format","doc":"The hash-tag `#` symbol denotes a comment when on its own line or when it follows a quoted value.  It is not treated as a comment when it appears within quotes.\n\n    # This is a comment\n    SECRET_KEY=YOURSECRETKEYGOESHERE # also a comment\n    SECRET_HASH=\"something-with-a-hash-#-this-is-not-a-comment\"","ref":"dotenv-file-format.html#comments"},{"type":"extras","title":"Command Substitution - Dotenv File Format","doc":"You need to add the output of a shell command in one of your variables? Simply add it with `$(your_command)` (available since version 1.0.0). This is helpful when tying into 3rd party password managers, e.g. [1Password](docs/guides/1password.md).\n\n    API_KEY=$(op read op://MyVault/SomeService/api_key)\n\nAs with interpolated variables, the `$()` syntax only triggers the execution of a shell command when it appears _within double-quotes or in a line without quotes_. If you quote your value with single quotes, the values will remain as-is.","ref":"dotenv-file-format.html#command-substitution"},{"type":"extras","title":"Configuration Providers","doc":"# Configuration Providers\n\n[Configuration providers](https://hexdocs.pm/elixir/Config.Provider.html) are most often invoked in the context of releases. Although they can solve certain problems that arise in production deployments, they tend to be an awkward fit for regular day-to-day development. `Dotenvy` seeks to normalize how configuration is loaded across environments, so having different strategies and functionality depending on how you run your app is antithetical. We do not want some code that runs only in certain environments and not in others: it can make for untested or untestable code!\n\nSecondly, configuration providers sometimes shift the task of \"shaping\" the configuration out of Elixir and into some static representation (e.g. JSON or TOML). The allure of a straight-forward static file is deceiving because there is no easy way to delineate Elixir-specific subtleties. such as distinguishing between keyword lists and maps. For example, how do you distinguish between a keyword list and a map in TOML?  Or how to differentiate a tuple and a list in JSON? And how can you indicate a map with string keys or atom keys?\n\nThe takeaway is that When configuration providers \"solve\" one problem, they often create another and mental friction accumulates.  It makes sense to handle problems about Elixir-specific data types inside Elixir instead of trying to cram a solution into some unsuspecting static file format.\n\nFor these reasons, `Dotenvy` does not rely on [configuration providers](https://hexdocs.pm/elixir/Config.Provider.html); `.env` files are an easier _lingua franca_.  As of `Dotenvy` version 1.0.0 and its support of shell commands, it's easier than ever to populate environment variables by using standard CLI tools to read values from password managers or other services.  See the page on [1Password](docs/1password.md) for and example.","ref":"configuration_providers.html"},{"type":"extras","title":"Generators","doc":"# Generators\n\nThe [Dotenvy Generators](https://hexdocs.pm/dotenvy_generators/) package includes mix tasks which are designed to spin up new Elixir applications that leverage `Dotenvy` to read environment variables. This is one of the easiest ways to see `Dotenvy` in action.\n\nThe most important tasks include:\n\n- `dot.new`: a variant of the humble `mix new` task\n- `phx.new`: an alternate of the Phoenix `mix phx.new` task, used to spin up Phoenix applications\n\n_The generators are still being developed, so they may in the process of being updated/improved._","ref":"generators.html"},{"type":"extras","title":"Installation - Generators","doc":"In order to install the `Dotenvy` generator scripts, you need to run two commands from your terminal: one to remove the `phx_new` generators (if present), and one to install the `dotenvy_generators`.  Run the following two commands:\n\n    mix archive.uninstall phx_new\n    mix archive.install hex dotenvy_generators\n\nOnce this has executed successfully, you should see the `dot.new` task as one of the available tasks when you run `mix help`.\n\nWe have to uninstall the `phx_new` generators because `dotenvy_generators` uses the same names for its mix tasks (this may change later... stay tuned).\n\nSee also the dedicated instructions in the [Dotenvy Generators](https://hexdocs.pm/dotenvy_generators/) package.","ref":"generators.html#installation"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.36.1"}}